/*****************************************************************************
 
            Copyright (c)2007 Geovariances, Avon, France.
 
    In consideration  of payment of  the license fee,  which is a part of
    the price you  paid for this  product, Geovariances (GV) as licensor,
    grants you, the licensee, a non-exclusive right to use this copy of a
    GV software product.
    GV reserves all rights not  expressly granted to licensee. GV retains
    titleship and ownership  of software.  This license is not  a sale of
    the original  software or any  copy. GV also  retains  titleship  and
    ownership of any modifications or  derivations of this software.  Any
    modifications of this software  must be clearly marked as such.  This
    copyright message must  appear in its entirety  in this software,  or
    any modifications or derivations thereof.
 
    Geovariances welcomes any comments, suggestions, bug reports, etc. At
    the discretion  of Geovariances,  any customer  supplied  bug  fixes,
    enhancements, or utility codes will be distributed in future software
    releases (the contributor will of course be credited).
 
            Geovariances
            49bis, Avenue Franklin Roosevelt
            77210 Avon, FRANCE
 
             Phone: +33-(0)-160.749.100
               Fax: +33-(0)-164.228.728
            e-mail: support@geovariances.fr
 
                        All Rights Reserved
 
*****************************************************************************/

#include <GTXClientP.h>

static char svnid[] _GTX_UNUSED = "$Id: api_get_study_list.c 11482 2008-01-04 20:37:41Z foucher $";

#include <string.h> /* strcpy() on solaris */

/*!
******************************************************************************
\brief Get study list in the Isatis file system

This function retrieve, from the GTXserver, the list of available
studies in the Isatis file system selected by the \ref GTXClientConnect call
\return error code:
  \arg 0 if Ok
  \arg 1 on error (see \ref GTXClientGetLastError)
\retval nb_study Number of studies in the Isatis file system
\retval studies Array of study names

\par Remarks:
Studies must be freed using
\code
studies = GTXClientFreeStringArray(nb_study, studies)
\endcode
*****************************************************************************/
int GTXClientGetStudyList(int *nb_study, char ***studies)
{
  int i;
  GTXErrorCode error;
  char answer[10];

  GTX_TRACE_FUNC_START("GTXClientGetStudyList",1);
  GTX_TRACE(1, ("(%p,%p)", nb_study, studies));
  *nb_study = 0;
  *studies = NULL;

  error = GTX_ERROR_NETWORK;
  if( _gtx_start_packet("CSTUDYLST") ) goto label_end;

  if( _gtx_send_and_wait(answer) ) goto label_end;

  error = GTX_ERROR_SERVER;
  if( _gtx_check_answer(answer, "SSTUDIES") ) goto label_end;

  error = GTX_ERROR_NETWORK;
  if( !_gtx_read_int(_gtx_server_socket, nb_study) ) goto label_end;

  error = GTX_ERROR_MEMORY;
  *studies = (char **)malloc(sizeof(char*) * (*nb_study));
  if( *studies == NULL ) goto label_end;

  error = GTX_ERROR_NETWORK;
  for(i=0; i<*nb_study; i++) (*studies)[i] = NULL;
  for(i=0; i<*nb_study; i++)
    if( !_gtx_read_string(_gtx_server_socket, &((*studies)[i]), 10000) ) goto label_end;

  error = GTX_ERROR_NONE;

label_end:
  if( error )
  {
    *studies = GTXClientFreeStringArray(*nb_study, *studies);
    *nb_study = 0;
    _gtx_cleanup_socket();
  }
  GTX_TRACE_FUNC_END("%d",error);
  _gtx_client_last_error = error;
  return (error != GTX_ERROR_NONE);
}
